@import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Sans:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap');
@import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Serif:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap');
@import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Mono:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap');

@media all {
    body, input {
        font-family: "IBM Plex Sans", "Fira Sans", Calibri, sans-serif;
        font-size: 1.05rem;
    }

    h1, h2, h3, h4, h5, h6 {
        font-family: "IBM Plex Serif", Georgia, Cambria, serif;
    }

    code, kbd, pre, textarea {
        font-family: "IBM Plex Mono", "Fira Code", Consolas, monospace;
        font-size: 1rem;
    }

    div.sphinxsidebar {
        font-size: 1rem;
    }

    p.caption, .caption-text {
        font-weight: bold;
    }

    .bolditalic {
        font-style: italic;
        font-weight: bold;
    }
}
